/**********************************************************************
File Name:             taskFlyport.c
Dev:                   Daniele De Santis
Email:                 desantis@areasx.com
Web:                   http://www.areasx.com

Versione:              0.0
Data:                  15/06/11
Note Hardware:  
Note Software: 
**********************************************************************/
#include "taskFlyport.h"

#define APP_NAME    "SX16 Wi-Fi"
#define APP_VER     "1.0"
#define APP_DATE    "08/07/2011"

//DEFINE GPIO
#define LED1_ON           IOPut(D3Out , 1)   //OUT3 su Net
#define LED1_OFF          IOPut(D3Out , 0)

#define LED2_ON           IOPut(D4Out , 1)   //OUT5 su Net - Led D2 su flyport
#define LED2_OFF          IOPut(D4Out , 0)
#define LED2_TOGGLE       IOPut(D4Out , IOGet(D4Out) ^ 0x01)

#define LED3_ON           IOPut(D5Out , 1)   //OUT4 su Net - Led D1 su flyport
#define LED3_OFF          IOPut(D5Out , 0)
#define LED3_TOGGLE       IOPut(D5Out , IOGet(D5Out) ^ 0x01)

#define BTN1_SETUP        IOInit(D2In,inup)  //IN2 su Net
#define BTN1_PRESS        (iobuttonstate(D2In)==pressed)


#define LINE_CLOCK       D5In
#define LINE_RELE_DATA   D4In
#define LINE_RELE_STROBE D3In
#define LINE_INPUT_DATA  D2In
#define LINE_INPUT_LOAD  D1In

#define CLOCK            IOPut(LINE_CLOCK,  1);IOPut(LINE_CLOCK ,  0)
#define RELE_STROBE      IOPut(LINE_RELE_STROBE, 1);IOPut(LINE_RELE_STROBE , 0)
#define RELE_SET_DATA    IOPut(LINE_RELE_DATA ,  1)
#define RELE_CLEAR_DATA  IOPut(LINE_RELE_DATA ,  0)
#define INPUT_LOAD       IOPut(LINE_INPUT_LOAD,  0);IOPut(LINE_INPUT_LOAD ,  1)
#define INPUT_GET_DATA   (IOGet(D2In))

//PROTOTIPI
void SetupLine();
void SetRele(int state);
unsigned long GetInput();
void DS1621_Init();
double DS1621_Read();

//Variabili Globali
typedef struct  {
  char ip[16];
  char nm[16]; 
  char gw[16];
  char ssid[30];
  BYTE mode;
  BYTE sec;
  char key[30];
} WEB_SETUP;

extern WEB_SETUP WEBconf;
double      temperature;
unsigned long input;
int 	    output;
char        meterdata[4];  //Primi 2 byte temperatura successivi umidita
extern BYTE WEBcmd;
char        app_ver[6];
char        app_date[11];
char        bytetemperature[2];


void SetupLine(){
  IOInit(LINE_CLOCK,out);
  IOInit(LINE_RELE_DATA,out);
  IOInit(LINE_RELE_STROBE,out);
  IOInit(LINE_INPUT_LOAD,out);
  IOInit(LINE_INPUT_DATA,in);
  IOPut(LINE_CLOCK,       0);
  IOPut(LINE_RELE_DATA ,  0);
  IOPut(LINE_RELE_STROBE, 0);
  IOPut(LINE_INPUT_LOAD,  1);
}

void SetRele(int state){
  int i;
  for (i=7;i>=0;i--) {
    if (state & (0x01<<i) ) 
      RELE_SET_DATA;
    else
      RELE_CLEAR_DATA;
    CLOCK;
    RELE_STROBE;
  }
}

unsigned long GetInput(){
  int i;
  unsigned long value;
  INPUT_LOAD;
  for (i=0;i<24;i++) {
    if (INPUT_GET_DATA) value |= 0x01;
    if (i!=23)value <<= 0x01;
    CLOCK;
  }
  return (16777215 - value);
}

void DS1621_Init(){
  I2CInit(LOW_SPEED);
  DelayMs(5);
  I2CRestart();
  DelayMs(5);
  I2CStart();
  I2CWrite(0x90);
  I2CWrite(0xAC);
  I2CWrite(0x2);
  I2CStart();
  I2CWrite(0x90);
  I2CWrite(0xEE);
  I2CStop();	
}

double DS1621_Read(){
  //char tmp[2];
  double temperature;
  I2CStart();
  I2CWrite(0x90);
  I2CWrite(0xAA);
  I2CStart();
  I2CWrite(0x91);
  bytetemperature[0]=I2CRead(0);
  DelayMs(5);
  bytetemperature[1]=I2CRead(1);
  I2CStop();
  temperature =0;
  if (bytetemperature[0] & 0x80) {
    temperature+=(!bytetemperature[0])+1;
    temperature=-temperature;
  } else temperature+=bytetemperature[0];
  if (bytetemperature[1]!=0x0) temperature+=0.5;
  return temperature;
}
//FINE FUNZIONI


void FlyportTask() {
  char        buffer[255];
  char        state;
  TCP_SOCKET  tcpid;
  DWORD       t0;
  DWORD       t1;
  char        debug[255];

  
  //Inizializzazione
  sprintf(app_ver,  "%s", APP_VER);
  sprintf(app_date, "%s", APP_DATE);
  //BTN1_SETUP;
  input       = 0;
  output      = 0;
  temperature = 0;
  
  state       = 0;
  t0=t1       = 0;
  WEBcmd      = 0;
  meterdata[0]=meterdata[1]=meterdata[2]=meterdata[3] = 0x00;
  

  if (WFCustomExist()) {

    UARTWrite(1,"\r\nConfigurazione Wi-Fi custom\r\n\r\n");
    LED1_OFF; LED2_OFF; LED3_OFF;
    WFCustomLoad ();
    
  } else {
    LED1_ON; LED2_ON; LED3_ON;
    UARTWrite(1,"\r\nConfigurazione Wi-Fi setup\r\n\r\n");
    WFSetParam(MY_IP_ADDR,          "192.168.1.100");
    WFSetParam(SUBNET_MASK,         "255.255.255.0");
    WFSetParam(DHCP_ENABLE,         ENABLED);
    WFSetParam(SSID_NAME,           "AREASX");
    WFSetParam(NETWORK_TYPE,        ADHOC);
    WFSetSecurity (WF_SECURITY_OPEN,"",0,0);
  }
  WFConnect(WF_CUSTOM);
  SetupLine();
  DS1621_Init();
  //attivo socket
  while(1) {
    if(WEBcmd){
      if(WEBcmd==1){  //LAN
        WFSetParam(MY_IP_ADDR,    WEBconf.ip);
        WFSetParam(SUBNET_MASK,   WEBconf.nm);
        WFSetParam(MY_GATEWAY,    WEBconf.gw);
        WFSetParam(DHCP_ENABLE,   DISABLED);
        WFCustomSave ();
        UARTWrite(1,"Do Setup LAN!\r\n");
      } else if (WEBcmd==2) { //WIFI
        WFSetParam(SSID_NAME,      WEBconf.ssid);
        if(WEBconf.mode){
          WFSetParam(NETWORK_TYPE, INFRASTRUCTURE);
        } else {
          WFSetParam(NETWORK_TYPE, ADHOC);
        }
        if(WEBconf.sec==0){  //OPEN
          WFSetSecurity (WF_SECURITY_OPEN,"",0,0);
        } else if(WEBconf.sec==2){ //WEP 128
          WFSetSecurity(WF_SECURITY_WEP_104,WEBconf.key,strlen(WEBconf.key),0);
        } else if(WEBconf.sec==4){ // WPA
          WFSetSecurity(WF_SECURITY_WPA_WITH_PASS_PHRASE,WEBconf.key,strlen(WEBconf.key),0);
        } else if(WEBconf.sec==6){ //WPA2
          WFSetSecurity(WF_SECURITY_WPA2_WITH_PASS_PHRASE,WEBconf.key,strlen(WEBconf.key),0);
        } else if(WEBconf.sec==8){ //AUTO WPA WPA2
          WFSetSecurity(WF_SECURITY_WPA_AUTO_WITH_PASS_PHRASE,WEBconf.key,strlen(WEBconf.key),0);
        }
        WFCustomSave ();
        UARTWrite(1,"Do Setup Wi-Fi\r\n");
      } else if (WEBcmd==3) { //REBOOT
        UARTWrite(1,"Do Reboot\r\n");
        vTaskDelay(50);
        exit(1); 
      } else if (WEBcmd==10) { //REBOOT
	    SetRele(output);
	  }
      WEBcmd=0;
    }

    switch(state){
      case 0:
       if(WFStatus != CONNECTED){ //Attendo connessione Wi-Fi
         LED3_TOGGLE;
         vTaskDelay(20);
       } else {
         LED1_OFF;LED2_OFF;
         state++;
       }
      break;
      case 1:
        tcpid=TCPServerOpen("1470");
        if(tcpid==INVALID_SOCKET) {
          state=0;
          vTaskDelay(20);
        } else{
          state++;
        }
      break;
      case 2:
        if(TCPisConn(tcpid)==FALSE){ //Attesa connessione
          LED2_TOGGLE;
          if(TickGet()-t1>10*TICK_SECOND){
            temperature = DS1621_Read();
			input       = GetInput();
            t1=TickGet();
          }
          state=4;
          continue;
        } else {
          t0=TickGet();
          state++;
        }
      break;
      
      case 3:
        if(TCPisConn(tcpid)){
          if(TickGet()-t0>60*TICK_SECOND){
            UARTWrite(1,"Connessione TCP/IP chiusa per timeout\r\n");
            TCPServerDetach(tcpid);
            state=4;
            continue;
          }
          if(TCPRxLen(tcpid)>=6){
            t0=TickGet();//Resetto timer timeout connessione
            LED1_ON;
            buffer[0]=0;
            TCPRead (tcpid, buffer, 6);
			
			sprintf(debug, "RX - %x %x %x %x %x %x\r\n",buffer[0],buffer[1],buffer[2],buffer[3],buffer[4],buffer[5]);
			UARTWrite(1,debug);
			
            if(((buffer[0]==0x66) || (buffer[0]==0x33)) && (buffer[1]==0x01)){
			  buffer[0]=0x55;
			  //buffer[1]=0x01;
			  //buffer[2]=0x00;
			  if(buffer[2]==0x01) { //Acquisizione stato ingressi
				input       =GetInput();     //Lettura ingressi
			    buffer[3]=(char)(input & 0xFF);
				buffer[4]=(char)((input>>8) & 0xFF);
				buffer[5]=(char)((input>>16) & 0xFF);
				t1=TickGet();  //Resetto timer acquisizione
			  } else if(buffer[2]==0x02) {
				temperature = DS1621_Read();  //Lettura temperatura
			    buffer[3]=bytetemperature[0];
				buffer[4]=bytetemperature[1];
				buffer[5]=0x00;
				t1=TickGet();  //Resetto timer acquisizione
			  } else if(buffer[2]==0x03) { //Set stato uscite
				//buffer[4] //Rele
				//buffer[5] //stato uscita
				if(((buffer[4]>0x00) || (buffer[4]<=0x06) || (buffer[4]==(char)0xFF)) && ((buffer[5]==0x00) || (buffer[5]==0x01))){
				  if(buffer[4]==(char)0xFF) {
					//UARTWrite(1,"Ingresso...\r\n");
				    if(buffer[5])
					  output=0x3F;
					else
					  output=0x00;
				  } else {
				    if(buffer[5])
					  output|=(0x01<<(buffer[4]-1));
					else
					  output&=~(0x01<<(buffer[4]-1));
				  }
				  SetRele(output);
			    }
			    buffer[3]=0x00;
				buffer[4]=(char)(output & 0xFF);
				buffer[5]=0x00;
			  } else if(buffer[2]==0x04) { //Acquisizione stato uscite
				buffer[3]=0x00;
				buffer[4]=(char)(output & 0xFF);
				buffer[5]=0x00;
			  } else if(buffer[2]==0x0B) {
				buffer[3]=0x01;
				buffer[4]=0x00;
				buffer[5]=0x00;
			  } else if(buffer[2]==0x0C) {
                buffer[3]=buffer[4]=buffer[5]=0x00;  
			  }
            } else {
		      buffer[1]=0xFF;
			  buffer[2]=0x00; 
			  buffer[3]=0x00; 
			  buffer[4]=0x00;  	
			}
            TCPWrite(tcpid, buffer, 6);
			sprintf(debug, "TX - %x %x %x %x %x %x\r\n",buffer[0],buffer[1],buffer[2],buffer[3],buffer[4],buffer[5]);
			UARTWrite(1,debug);
			
            LED1_OFF;
          } else {
            vTaskDelay(5); //1/2 secondo
          }
        } else state=4;
      break;
      
      case 4:
        if(WFStatus != CONNECTED) 
          state=0;
        else {
          state=2;
          vTaskDelay(20);
        }
      break;
      
    }
  }
}
